@@ -16,21 +16,25 @@ email                : marco.hugentobler at sourcepole dot com
 #ifndef QGSABSTRACTGEOMETRYV2
 #define QGSABSTRACTGEOMETRYV2
 
+#include <array>
 #include <functional>
-
 #include <QString>
 
 #include "qgis_core.h"
-#include "qgis.h"
 #include "qgscoordinatetransform.h"
 #include "qgswkbtypes.h"
 #include "qgswkbptr.h"
 
+#ifndef SIP_RUN
+#include "json_fwd.hpp"
+using namespace nlohmann;
+#endif
+
 class QgsMapToPixel;
 class QgsCurve;
 class QgsMultiCurve;
 class QgsMultiPoint;
-class QgsPoint;
+
 struct QgsVertexId;
 class QgsVertexIterator;
 class QPainter;
@@ -38,6 +42,8 @@ class QDomDocument;
 class QDomElement;
 class QgsGeometryPartIterator;
 class QgsGeometryConstPartIterator;
+class QgsConstWkbPtr;
+class QPainterPath;
 
 typedef QVector< QgsPoint > QgsPointSequence;
 #ifndef SIP_RUN
@@ -48,10 +54,20 @@ typedef QVector< QVector< QgsPoint > > QgsRingSequence;
 typedef QVector< QVector< QVector< QgsPoint > > > QgsCoordinateSequence;
 #endif
 
+
 /**
  * \ingroup core
  * \class QgsAbstractGeometry
  * \brief Abstract base class for all geometries
+ *
+ * \note QgsAbstractGeometry objects are inherently Cartesian/planar geometries. They have no concept of geodesy, and none
+ * of the methods or properties exposed from the QgsAbstractGeometry API (or QgsGeometry API) utilize
+ * geodesic calculations. Accordingly, properties like length() and area() and spatial operations like centroid()
+ * are always calculated using strictly Cartesian mathematics. In contrast, the QgsDistanceArea class exposes
+ * methods for working with geodesic calculations and spatial operations on geometries,
+ * and should be used whenever calculations which account for the curvature of the Earth (or any other celestial body)
+ * are required.
+ *
  * \since QGIS 2.10
  */
 class CORE_EXPORT QgsAbstractGeometry
@@ -100,12 +116,14 @@ class CORE_EXPORT QgsAbstractGeometry
 
       /**
        * Maximum angle between generating radii (lines from arc center
-       * to output vertices) */
+       * to output vertices)
+      */
       MaximumAngle = 0,
 
       /**
        * Maximum distance between an arbitrary point on the original
-       * curve and closest point on its approximation. */
+       * curve and closest point on its approximation.
+      */
       MaximumDifference
     };
     Q_ENUM( SegmentationToleranceType )
@@ -172,7 +190,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * \see geometryType
      * \see wktTypeStr
      */
-    inline QgsWkbTypes::Type wkbType() const { return mWkbType; }
+    inline QgsWkbTypes::Type wkbType() const SIP_HOLDGIL { return mWkbType; }
 
     /**
      * Returns the WKT type string of the geometry.
@@ -182,19 +200,19 @@ class CORE_EXPORT QgsAbstractGeometry
     QString wktTypeStr() const;
 
     /**
-     * Returns true if the geometry is 3D and contains a z-value.
+     * Returns TRUE if the geometry is 3D and contains a z-value.
      * \see isMeasure
      */
-    bool is3D() const
+    bool is3D() const SIP_HOLDGIL
     {
       return QgsWkbTypes::hasZ( mWkbType );
     }
 
     /**
-     * Returns true if the geometry contains m values.
+     * Returns TRUE if the geometry contains m values.
      * \see is3D
      */
-    bool isMeasure() const
+    bool isMeasure() const SIP_HOLDGIL
     {
       return QgsWkbTypes::hasM( mWkbType );
     }
@@ -202,7 +220,7 @@ class CORE_EXPORT QgsAbstractGeometry
     /**
      * Returns the closure of the combinatorial boundary of the geometry (ie the topological boundary of the geometry).
      * For instance, a polygon geometry will have a boundary consisting of the linestrings for each ring in the polygon.
-     * \returns boundary for geometry. May be null for some geometry types.
+     * \returns boundary for geometry. May be NULLPTR for some geometry types.
      * \since QGIS 3.0
      */
     virtual QgsAbstractGeometry *boundary() const = 0 SIP_FACTORY;
@@ -224,15 +242,37 @@ class CORE_EXPORT QgsAbstractGeometry
 
     //export
 
+    /**
+     * WKB export flags.
+     * \since QGIS 3.14
+     */
+    enum WkbFlag
+    {
+      FlagExportTrianglesAsPolygons = 1 << 0, //!< Triangles should be exported as polygon geometries
+    };
+    Q_DECLARE_FLAGS( WkbFlags, WkbFlag )
+
+    /**
+     * Returns the length of the QByteArray returned by asWkb()
+     *
+     * The optional \a flags argument specifies flags controlling WKB export behavior
+     *
+     * \since QGIS 3.16
+     */
+    virtual int wkbSize( QgsAbstractGeometry::WkbFlags flags = QgsAbstractGeometry::WkbFlags() ) const = 0;
+
     /**
      * Returns a WKB representation of the geometry.
+     *
+     * The optional \a flags argument specifies flags controlling WKB export behavior (since QGIS 3.14).
+     *
      * \see asWkt
      * \see asGml2
      * \see asGml3
      * \see asJson()
      * \since QGIS 3.0
      */
-    virtual QByteArray asWkb() const = 0;
+    virtual QByteArray asWkb( WkbFlags flags = QgsAbstractGeometry::WkbFlags() ) const = 0;
 
     /**
      * Returns a WKT representation of the geometry.
@@ -271,14 +311,34 @@ class CORE_EXPORT QgsAbstractGeometry
     virtual QDomElement asGml3( QDomDocument &doc, int precision = 17, const QString &ns = "gml", AxisOrder axisOrder = QgsAbstractGeometry::AxisOrder::XY ) const = 0;
 
     /**
-     * Returns a GeoJSON representation of the geometry.
+     * Returns a GeoJSON representation of the geometry as a QString.
      * \param precision number of decimal places for coordinates
      * \see asWkb()
      * \see asWkt()
      * \see asGml2()
      * \see asGml3()
+     * \see asJsonObject()
      */
-    virtual QString asJson( int precision = 17 ) const = 0;
+    QString asJson( int precision = 17 );
+
+    /**
+     * Returns a json object representation of the geometry.
+     * \see asWkb()
+     * \see asWkt()
+     * \see asGml2()
+     * \see asGml3()
+     * \see asJson()
+     * \note not available in Python bindings
+     * \since QGIS 3.10
+     */
+    virtual json asJsonObject( int precision = 17 ) SIP_SKIP const;
+
+    /**
+     * Returns a KML representation of the geometry.
+     * \since QGIS 3.12
+     */
+    virtual QString asKml( int precision = 17 ) const = 0;
+
 
     //render pipeline
 
@@ -286,10 +346,10 @@ class CORE_EXPORT QgsAbstractGeometry
      * Transforms the geometry using a coordinate transform
      * \param ct coordinate transform
      * \param d transformation direction
-     * \param transformZ set to true to also transform z coordinates. This requires that
+     * \param transformZ set to TRUE to also transform z coordinates. This requires that
      * the z coordinates in the geometry represent height relative to the vertical datum
      * of the source CRS (generally ellipsoidal heights) and are expressed in its vertical
-     * units (generally meters). If false, then z coordinates will not be changed by the
+     * units (generally meters). If FALSE, then z coordinates will not be changed by the
      * transform.
      */
     virtual void transform( const QgsCoordinateTransform &ct, QgsCoordinateTransform::TransformDirection d = QgsCoordinateTransform::ForwardTransform, bool transformZ = false ) SIP_THROW( QgsCsException ) = 0;
@@ -309,6 +369,16 @@ class CORE_EXPORT QgsAbstractGeometry
      */
     virtual void draw( QPainter &p ) const = 0;
 
+    /**
+     * Returns the geometry represented as a QPainterPath.
+     *
+     * \warning not all geometry subclasses can be represented by a QPainterPath, e.g.
+     * points and multipoint geometries will return an empty path.
+     *
+     * \since QGIS 3.16
+     */
+    virtual QPainterPath asQPainterPath() const = 0;
+
     /**
      * Returns the vertex number corresponding to a vertex \a id.
      *
@@ -326,7 +396,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * \param id initial value should be the starting vertex id. The next vertex id will be stored
      * in this variable if found.
      * \param vertex container for found node
-     * \returns false if at end
+     * \returns FALSE if at end
      */
     virtual bool nextVertex( QgsVertexId &id, QgsPoint &vertex SIP_OUT ) const = 0;
 
@@ -360,7 +430,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * \param leftOf indicates whether the point lies on the left side of the geometry (-1 if point is to the left
      * of the geometry, +1 if the point is to the right of the geometry, or 0 for cases where left/right could not
      * be determined, e.g. point exactly on a line)
-     * false if point is to right of segment)
+     * FALSE if point is to right of segment)
      * \param epsilon epsilon for segment snapping
      * \returns squared distance to closest segment or negative value on error
      */
@@ -374,7 +444,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * Inserts a vertex into the geometry
      * \param position vertex id for position of inserted vertex
      * \param vertex vertex to insert
-     * \returns true if insert was successful
+     * \returns TRUE if insert was successful
      * \see moveVertex
      * \see deleteVertex
      */
@@ -384,7 +454,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * Moves a vertex within the geometry
      * \param position vertex id for vertex to move
      * \param newPos new position of vertex
-     * \returns true if move was successful
+     * \returns TRUE if move was successful
      * \see insertVertex
      * \see deleteVertex
      */
@@ -393,28 +463,49 @@ class CORE_EXPORT QgsAbstractGeometry
     /**
      * Deletes a vertex within the geometry
      * \param position vertex id for vertex to delete
-     * \returns true if delete was successful
+     * \returns TRUE if delete was successful
      * \see insertVertex
      * \see moveVertex
      */
     virtual bool deleteVertex( QgsVertexId position ) = 0;
 
     /**
-     * Returns the length of the geometry.
+     * Returns the planar, 2-dimensional length of the geometry.
+     *
+     * \warning QgsAbstractGeometry objects are inherently Cartesian/planar geometries, and the length
+     * returned by this method is calculated using strictly Cartesian mathematics. In contrast,
+     * the QgsDistanceArea class exposes methods for calculating the lengths of geometries using
+     * geodesic calculations which account for the curvature of the Earth (or any other
+     * celestial body).
+     *
      * \see area()
      * \see perimeter()
      */
     virtual double length() const;
 
     /**
-     * Returns the perimeter of the geometry.
+     * Returns the planar, 2-dimensional perimeter of the geometry.
+     *
+     * \warning QgsAbstractGeometry objects are inherently Cartesian/planar geometries, and the perimeter
+     * returned by this method is calculated using strictly Cartesian mathematics. In contrast,
+     * the QgsDistanceArea class exposes methods for calculating the perimeters of geometries using
+     * geodesic calculations which account for the curvature of the Earth (or any other
+     * celestial body).
+     *
      * \see area()
      * \see length()
      */
     virtual double perimeter() const;
 
     /**
-     * Returns the area of the geometry.
+     * Returns the planar, 2-dimensional area of the geometry.
+     *
+     * \warning QgsAbstractGeometry objects are inherently Cartesian/planar geometries, and the area
+     * returned by this method is calculated using strictly Cartesian mathematics. In contrast,
+     * the QgsDistanceArea class exposes methods for calculating the areas of geometries using
+     * geodesic calculations which account for the curvature of the Earth (or any other
+     * celestial body).
+     *
      * \see length()
      * \see perimeter()
      */
@@ -422,6 +513,10 @@ class CORE_EXPORT QgsAbstractGeometry
 
     /**
      * Returns the length of the segment of the geometry which begins at \a startVertex.
+     *
+     * \warning QgsAbstractGeometry objects are inherently Cartesian/planar geometries, and the lengths
+     * returned by this method are calculated using strictly Cartesian mathematics.
+     *
      * \since QGIS 3.0
      */
     virtual double segmentLength( QgsVertexId startVertex ) const = 0;
@@ -430,12 +525,12 @@ class CORE_EXPORT QgsAbstractGeometry
     virtual QgsPoint centroid() const;
 
     /**
-     * Returns true if the geometry is empty
+     * Returns TRUE if the geometry is empty
      */
     virtual bool isEmpty() const;
 
     /**
-     * Returns true if the geometry contains curved segments
+     * Returns TRUE if the geometry contains curved segments
      */
     virtual bool hasCurvedSegments() const;
 
@@ -459,13 +554,16 @@ class CORE_EXPORT QgsAbstractGeometry
      * Makes a new geometry with all the points or vertices snapped to the closest point of the grid.
      * Ownership is transferred to the caller.
      *
-     * If the gridified geometry could not be calculated a nullptr will be returned.
+     * If the gridified geometry could not be calculated NULLPTR will be returned.
      * It may generate an invalid geometry (in some corner cases).
      * It can also be thought as rounding the edges and it may be useful for removing errors.
+     *
      * Example:
-     * \code{.cpp}
-     * geometry->snappedToGrid(1, 1);
+     *
+     * \code{.py}
+     *   geometry.snappedToGrid(1, 1)
      * \endcode
+     *
      * In this case we use a 2D grid of 1x1 to gridify.
      * In this case, it can be thought like rounding the x and y of all the points/vertices to full units (remove all decimals).
      * \param hSpacing Horizontal spacing of the grid (x axis). 0 to disable.
@@ -485,13 +583,13 @@ class CORE_EXPORT QgsAbstractGeometry
      *
      * By default, z values are not considered when detecting duplicate nodes. E.g. two nodes
      * with the same x and y coordinate but different z values will still be considered
-     * duplicate and one will be removed. If \a useZValues is true, then the z values are
+     * duplicate and one will be removed. If \a useZValues is TRUE, then the z values are
      * also tested and nodes with the same x and y but different z will be maintained.
      *
      * Note that duplicate nodes are not tested between different parts of a multipart geometry. E.g.
      * a multipoint geometry with overlapping points will not be changed by this method.
      *
-     * The function will return true if nodes were removed, or false if no duplicate nodes
+     * The function will return TRUE if nodes were removed, or FALSE if no duplicate nodes
      * were found.
      *
      * \since QGIS 3.0
@@ -527,7 +625,7 @@ class CORE_EXPORT QgsAbstractGeometry
     /**
      * Adds a z-dimension to the geometry, initialized to a preset value.
      * \param zValue initial z-value for all nodes
-     * \returns true on success
+     * \returns TRUE on success
      * \see dropZValue()
      * \see addMValue()
      * \since QGIS 2.12
@@ -537,7 +635,7 @@ class CORE_EXPORT QgsAbstractGeometry
     /**
      * Adds a measure to the geometry, initialized to a preset value.
      * \param mValue initial m-value for all nodes
-     * \returns true on success
+     * \returns TRUE on success
      * \see dropMValue()
      * \see addZValue()
      * \since QGIS 2.12
@@ -546,7 +644,7 @@ class CORE_EXPORT QgsAbstractGeometry
 
     /**
      * Drops any z-dimensions which exist in the geometry.
-     * \returns true if Z values were present and have been removed
+     * \returns TRUE if Z values were present and have been removed
      * \see addZValue()
      * \see dropMValue()
      * \since QGIS 2.14
@@ -555,7 +653,7 @@ class CORE_EXPORT QgsAbstractGeometry
 
     /**
      * Drops any measure values which exist in the geometry.
-     * \returns true if m-values were present and have been removed
+     * \returns TRUE if m-values were present and have been removed
      * \see addMValue()
      * \see dropZValue()
      * \since QGIS 2.14
@@ -572,15 +670,28 @@ class CORE_EXPORT QgsAbstractGeometry
 
     /**
      * Converts the geometry to a specified type.
-     * \returns true if conversion was successful
+     * \returns TRUE if conversion was successful
      * \since QGIS 2.14
      */
     virtual bool convertTo( QgsWkbTypes::Type type );
 
+    /**
+     * Checks validity of the geometry, and returns TRUE if the geometry is valid.
+     *
+     * \param error will be set to the validity error message
+     * \param flags indicates optional flags which control the type of validity checking performed
+     * (corresponding to QgsGeometry::ValidityFlags).
+     *
+     * \returns TRUE if geometry is valid
+     *
+     * \since QGIS 3.8
+     */
+    virtual bool isValid( QString &error SIP_OUT, int flags = 0 ) const = 0;
+
 #ifndef SIP_RUN
 
     /**
-     * Filters the vertices from the geometry in place, removing any which do not return true for the \a filter function
+     * Filters the vertices from the geometry in place, removing any which do not return TRUE for the \a filter function
      * check. Has no meaning when called on a single point geometry.
      *
      * Depending on the \a filter used, this may result in an invalid geometry.
@@ -609,7 +720,7 @@ class CORE_EXPORT QgsAbstractGeometry
     /**
      * \ingroup core
      * The part_iterator class provides STL-style iterator for geometry parts.
-     * \since QGIS 3.4.3
+     * \since QGIS 3.6
      */
     class CORE_EXPORT part_iterator
     {
@@ -650,7 +761,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * \see parts_end()
      * \see parts()
      *
-     * \since QGIS 3.4.3
+     * \since QGIS 3.6
      */
     part_iterator parts_begin()
     {
@@ -663,7 +774,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * \see parts_begin()
      * \see parts()
      *
-     * \since QGIS 3.4.3
+     * \since QGIS 3.6
      */
     part_iterator parts_end();
 
@@ -672,14 +783,14 @@ class CORE_EXPORT QgsAbstractGeometry
      * returns read-only references to parts and cannot be used to modify the parts.
      *
      * \note Not available in Python bindings
-     * \since QGIS 3.4.3
+     * \since QGIS 3.6
      */
     QgsGeometryConstPartIterator parts() const;
 
     /**
      * \ingroup core
      * The part_iterator class provides STL-style iterator for const references to geometry parts.
-     * \since QGIS 3.4.3
+     * \since QGIS 3.6
      */
     class CORE_EXPORT const_part_iterator
     {
@@ -719,7 +830,7 @@ class CORE_EXPORT QgsAbstractGeometry
      *
      * \see const_parts_end()
      *
-     * \since QGIS 3.4.3
+     * \since QGIS 3.6
      */
     const_part_iterator const_parts_begin() const
     {
@@ -731,7 +842,7 @@ class CORE_EXPORT QgsAbstractGeometry
      *
      * \see const_parts_begin()
      *
-     * \since QGIS 3.4.3
+     * \since QGIS 3.6
      */
     const_part_iterator const_parts_end() const;
 
@@ -754,10 +865,12 @@ class CORE_EXPORT QgsAbstractGeometry
         {
           const QgsAbstractGeometry *g = nullptr;  //!< Current geometry
           int index = 0;               //!< Ptr in the current geometry
+
+          bool operator==( const Level &other ) const;
         };
 
-        Level levels[3];  //!< Stack of levels - three levels should be sufficient (e.g. part index, ring index, vertex index)
-        int depth = -1;        //!< At what depth level are we right now
+        std::array<Level, 3> levels;  //!< Stack of levels - three levels should be sufficient (e.g. part index, ring index, vertex index)
+        int depth = -1;               //!< At what depth level are we right now
 
         void digDown();   //!< Prepare the stack of levels so that it points to a leaf child geometry
 
@@ -818,7 +931,8 @@ class CORE_EXPORT QgsAbstractGeometry
      * Returns Java-style iterator for traversal of parts of the geometry. This iterator
      * can safely be used to modify parts of the geometry.
      *
-     * * Example:
+     * Example
+     *
      * \code{.py}
      *   # print the WKT representation of each part in a multi-point geometry
      *   geometry = QgsMultiPoint.fromWkt( 'MultiPoint( 0 0, 1 1, 2 2)' )
@@ -844,7 +958,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * \endcode
      *
      * \see vertices()
-     * \since QGIS 3.4.3
+     * \since QGIS 3.6
      */
     QgsGeometryPartIterator parts();
 
@@ -855,7 +969,8 @@ class CORE_EXPORT QgsAbstractGeometry
      * \warning The iterator returns a copy of individual vertices, and accordingly geometries cannot be
      * modified using the iterator. See transformVertices() for a safe method to modify vertices "in-place".
      *
-     * * Example:
+     * Example
+     *
      * \code{.py}
      *   # print the x and y coordinate for each vertex in a LineString
      *   geometry = QgsLineString.fromWkt( 'LineString( 0 0, 1 1, 2 2)' )
@@ -884,7 +999,7 @@ class CORE_EXPORT QgsAbstractGeometry
   protected:
 
     /**
-     * Returns whether the geometry has any child geometries (false for point / curve, true otherwise)
+     * Returns whether the geometry has any child geometries (FALSE for point / curve, TRUE otherwise)
      * \note used for vertex_iterator implementation
      * \since QGIS 3.0
      */
@@ -902,7 +1017,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * \note used for vertex_iterator implementation
      * \since QGIS 3.0
      */
-    virtual QgsAbstractGeometry *childGeometry( int index ) const { Q_UNUSED( index ); return nullptr; }
+    virtual QgsAbstractGeometry *childGeometry( int index ) const { Q_UNUSED( index ) return nullptr; }
 
     /**
      * Returns point at index (for geometries without child geometries - i.e. curve / point)
@@ -942,55 +1057,96 @@ class CORE_EXPORT QgsAbstractGeometry
  */
 struct CORE_EXPORT QgsVertexId
 {
+
+  /**
+   * Type of vertex
+   */
   enum VertexType
   {
-    SegmentVertex = 1, //start / endpoint of a segment
-    CurveVertex
+    SegmentVertex = 1, //!< The actual start or end point of a segment
+    CurveVertex, //!< An intermediate point on a segment defining the curvature of the segment
   };
 
-  explicit QgsVertexId( int _part = -1, int _ring = -1, int _vertex = -1, VertexType _type = SegmentVertex )
-    : part( _part )
-    , ring( _ring )
-    , vertex( _vertex )
-    , type( _type )
+  /**
+   * Constructor for QgsVertexId.
+   */
+  explicit QgsVertexId( int _part = -1, int _ring = -1, int _vertex = -1, VertexType _type = SegmentVertex ) SIP_HOLDGIL
+: part( _part )
+  , ring( _ring )
+  , vertex( _vertex )
+  , type( _type )
   {}
 
   /**
-   * Returns true if the vertex id is valid
+   * Returns TRUE if the vertex id is valid
    */
-  bool isValid() const { return part >= 0 && ring >= 0 && vertex >= 0; }
+  bool isValid() const  SIP_HOLDGIL { return part >= 0 && ring >= 0 && vertex >= 0; }
 
-  bool operator==( QgsVertexId other ) const
+  bool operator==( QgsVertexId other ) const SIP_HOLDGIL
   {
     return part == other.part && ring == other.ring && vertex == other.vertex;
   }
-  bool operator!=( QgsVertexId other ) const
+  bool operator!=( QgsVertexId other ) const SIP_HOLDGIL
   {
     return part != other.part || ring != other.ring || vertex != other.vertex;
   }
-  bool partEqual( QgsVertexId o ) const
+
+  /**
+   * Returns TRUE if this vertex ID belongs to the same part as another vertex ID.
+   */
+  bool partEqual( QgsVertexId o ) const SIP_HOLDGIL
   {
     return part >= 0 && o.part == part;
   }
-  bool ringEqual( QgsVertexId o ) const
+
+  /**
+   * Returns TRUE if this vertex ID belongs to the same ring as another vertex ID (i.e. the part
+   * and ring number are equal).
+   */
+  bool ringEqual( QgsVertexId o ) const SIP_HOLDGIL
   {
     return partEqual( o ) && ( ring >= 0 && o.ring == ring );
   }
-  bool vertexEqual( QgsVertexId o ) const
+
+  /**
+   * Returns TRUE if this vertex ID corresponds to the same vertex as another vertex ID (i.e. the part,
+   * ring number and vertex number are equal).
+   */
+  bool vertexEqual( QgsVertexId o ) const SIP_HOLDGIL
   {
     return ringEqual( o ) && ( vertex >= 0 && o.ring == ring );
   }
-  bool isValid( const QgsAbstractGeometry *geom ) const
+
+  /**
+   * Returns TRUE if this vertex ID is valid for the specified \a geom.
+   */
+  bool isValid( const QgsAbstractGeometry *geom ) const SIP_HOLDGIL
   {
     return ( part >= 0 && part < geom->partCount() ) &&
            ( ring < geom->ringCount( part ) ) &&
            ( vertex < 0 || vertex < geom->vertexCount( part, ring ) );
   }
 
-  int part;
-  int ring;
-  int vertex;
-  VertexType type;
+  //! Part number
+  int part = -1;
+
+  //! Ring number
+  int ring = -1;
+
+  //! Vertex number
+  int vertex = -1;
+
+  //! Vertex type
+  VertexType type = SegmentVertex;
+
+#ifdef SIP_RUN
+  SIP_PYOBJECT __repr__();
+  % MethodCode
+  QString str = QStringLiteral( "<QgsVertexId: %1,%2,%3%4>" ).arg( sipCpp->part ).arg( sipCpp->ring ).arg( sipCpp->vertex ).arg( sipCpp->type == QgsVertexId::CurveVertex ? QStringLiteral( " CurveVertex" ) : QString() );
+  sipRes = PyUnicode_FromString( str.toUtf8().data() );
+  % End
+#endif
+
 };
 
 #ifndef SIP_RUN
@@ -1030,7 +1186,7 @@ class CORE_EXPORT QgsVertexIterator
       return g && g->vertices_end() != i;
     }
 
-    //! Returns next vertex of the geometry (undefined behavior if hasNext() returns false before calling next())
+    //! Returns next vertex of the geometry (undefined behavior if hasNext() returns FALSE before calling next())
     QgsPoint next();
 
 #ifdef SIP_RUN
@@ -1039,7 +1195,7 @@ class CORE_EXPORT QgsVertexIterator
     sipRes = sipCpp;
     % End
 
-    SIP_PYOBJECT __next__();
+    SIP_PYOBJECT __next__() SIP_TYPEHINT( QgsPoint );
     % MethodCode
     if ( sipCpp->hasNext() )
       sipRes = sipConvertFromType( new QgsPoint( sipCpp->next() ), sipType_QgsPoint, Py_None );
@@ -1057,7 +1213,7 @@ class CORE_EXPORT QgsVertexIterator
 /**
  * \ingroup core
  * \brief Java-style iterator for traversal of parts of a geometry
- * \since QGIS 3.4.3
+ * \since QGIS 3.6
  */
 class CORE_EXPORT QgsGeometryPartIterator
 {
@@ -1074,12 +1230,12 @@ class CORE_EXPORT QgsGeometryPartIterator
     }
 
     //! Find out whether there are more parts
-    bool hasNext() const
+    bool hasNext() const SIP_HOLDGIL
     {
       return g && g->parts_end() != i;
     }
 
-    //! Returns next part of the geometry (undefined behavior if hasNext() returns false before calling next())
+    //! Returns next part of the geometry (undefined behavior if hasNext() returns FALSE before calling next())
     QgsAbstractGeometry *next();
 
 #ifdef SIP_RUN
@@ -1088,7 +1244,7 @@ class CORE_EXPORT QgsGeometryPartIterator
     sipRes = sipCpp;
     % End
 
-    SIP_PYOBJECT __next__();
+    SIP_PYOBJECT __next__() SIP_TYPEHINT( QgsAbstractGeometry );
     % MethodCode
     if ( sipCpp->hasNext() )
       sipRes = sipConvertFromType( sipCpp->next(), sipType_QgsAbstractGeometry, NULL );
@@ -1107,7 +1263,7 @@ class CORE_EXPORT QgsGeometryPartIterator
 /**
  * \ingroup core
  * \brief Java-style iterator for const traversal of parts of a geometry
- * \since QGIS 3.4.3
+ * \since QGIS 3.6
  */
 class CORE_EXPORT QgsGeometryConstPartIterator
 {
@@ -1124,12 +1280,12 @@ class CORE_EXPORT QgsGeometryConstPartIterator
     }
 
     //! Find out whether there are more parts
-    bool hasNext() const
+    bool hasNext() const SIP_HOLDGIL
     {
       return g && g->const_parts_end() != i;
     }
 
-    //! Returns next part of the geometry (undefined behavior if hasNext() returns false before calling next())
+    //! Returns next part of the geometry (undefined behavior if hasNext() returns FALSE before calling next())
     const QgsAbstractGeometry *next();
 
 #ifdef SIP_RUN
@@ -1138,7 +1294,7 @@ class CORE_EXPORT QgsGeometryConstPartIterator
     sipRes = sipCpp;
     % End
 
-    SIP_PYOBJECT __next__();
+    SIP_PYOBJECT __next__() SIP_TYPEHINT( QgsAbstractGeometry );
     % MethodCode
     if ( sipCpp->hasNext() )
       sipRes = sipConvertFromType( const_cast< QgsAbstractGeometry * >( sipCpp->next() ), sipType_QgsAbstractGeometry, NULL );
@@ -1153,4 +1309,6 @@ class CORE_EXPORT QgsGeometryConstPartIterator
 
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsAbstractGeometry::WkbFlags )
+
 #endif //QGSABSTRACTGEOMETRYV2
