@@ -32,7 +32,8 @@ class QgsBrowserDockWidget;
 class QgsRasterLayer;
 class QgsMapCanvas;
 class QgsAbstractDataSourceWidget;
-class QgsBrowserModel;
+class QgsBrowserGuiModel;
+class QgsMessageBar;
 
 /**
  * \ingroup gui
@@ -56,7 +57,7 @@ class GUI_EXPORT QgsDataSourceManagerDialog : public QgsOptionsDialogBase, priva
       * \param canvas a pointer to the map canvas
       * \param fl window flags
       */
-    explicit QgsDataSourceManagerDialog( QgsBrowserModel *browserModel, QWidget *parent = nullptr, QgsMapCanvas *canvas = nullptr, Qt::WindowFlags fl = QgsGuiUtils::ModalDialogFlags );
+    explicit QgsDataSourceManagerDialog( QgsBrowserGuiModel *browserModel, QWidget *parent = nullptr, QgsMapCanvas *canvas = nullptr, Qt::WindowFlags fl = QgsGuiUtils::ModalDialogFlags );
     ~QgsDataSourceManagerDialog() override;
 
     /**
@@ -66,14 +67,18 @@ class GUI_EXPORT QgsDataSourceManagerDialog : public QgsOptionsDialogBase, priva
      */
     void openPage( const QString &pageName );
 
+    //! Returns the dialog's message bar
+    QgsMessageBar *messageBar() const;
+
   public slots:
 
     //! Sync current page with the leftbar list
     void setCurrentPage( int index );
 
+    // TODO: use this with an internal source select dialog instead of forwarding the whole raster selection to app
+
     /**
      * A raster layer was added: for signal forwarding to QgisApp
-     * TODO: use this with an internal source select dialog instead of forwarding the whole raster selection to app
      */
     void rasterLayerAdded( QString const &uri, QString const &baseName, QString const &providerKey );
     //! A vector layer was added: for signal forwarding to QgisApp
@@ -85,6 +90,15 @@ class GUI_EXPORT QgsDataSourceManagerDialog : public QgsOptionsDialogBase, priva
     //! Refresh the browser view
     void refresh();
 
+    /**
+     * Resets the interface of the datasource manager after reopening the dialog.
+     *
+     * Will clear the selection of embedded all source selection widgets.
+     *
+     * \since QGIS 3.10
+     */
+    void reset();
+
   protected:
     void showEvent( QShowEvent *event ) override;
 
@@ -95,12 +109,24 @@ class GUI_EXPORT QgsDataSourceManagerDialog : public QgsOptionsDialogBase, priva
     void addRasterLayer();
     //! Emitted when a vector layer was selected for addition: for signal forwarding to QgisApp
     void addVectorLayer( const QString &vectorLayerPath, const QString &baseName, const QString &providerKey );
+
+    /**
+     * Emitted when a mesh layer was selected for addition: for signal forwarding to QgisApp
+     * \since QGIS 3.4
+     */
+    void addMeshLayer( const QString &uri, const QString &baseName, const QString &providerKey );
+
+    /**
+     * Emitted when a vector tile layer was selected for addition: for signal forwarding to QgisApp
+     * \since QGIS 3.14
+     */
+    void addVectorTileLayer( const QString &uri, const QString &baseName );
+
     //! Replace the selected layer by a vector layer defined by uri, layer name, data source uri
     void replaceSelectedVectorLayer( const QString &oldId, const QString &uri, const QString &layerName, const QString &provider );
     //! Emitted when a one or more layer were selected for addition: for signal forwarding to QgisApp
     void addVectorLayers( const QStringList &layerQStringList, const QString &enc, const QString &dataSourceType );
-    //! Emitted when the dialog is busy: for signal forwarding to QgisApp
-    void showProgress( int progress, int totalSteps );
+
     //! Emitted when a status message needs to be shown: for signal forwarding to QgisApp
     void showStatusMessage( const QString &message );
     //! Emitted when a DB layer was selected for addition: for signal forwarding to QgisApp
@@ -134,7 +160,7 @@ class GUI_EXPORT QgsDataSourceManagerDialog : public QgsOptionsDialogBase, priva
     QStringList mPageNames;
     // Map canvas
     QgsMapCanvas *mMapCanvas = nullptr;
-
+    QgsMessageBar *mMessageBar = nullptr;
 
 };
 
