@@ -102,6 +102,12 @@ struct CORE_EXPORT QgsMesh
     */
   void clear();
 
+  /**
+   * Compare two faces, return TRUE if they are equivalent : same indexes and same clock wise
+    * \since QGIS 3.16
+   */
+  static bool compareFaces( const QgsMeshFace &face1, const QgsMeshFace &face2 );
+
   QVector<QgsMeshVertex> vertices SIP_SKIP;
   QVector<QgsMeshEdge> edges SIP_SKIP;
   QVector<QgsMeshFace> faces SIP_SKIP;
@@ -182,6 +188,7 @@ class CORE_EXPORT QgsMeshDataSourceInterface SIP_ABSTRACT
 class CORE_EXPORT QgsMeshDatasetSourceInterface SIP_ABSTRACT
 {
   public:
+    QgsMeshDatasetSourceInterface();
     //! Dtor
     virtual ~QgsMeshDatasetSourceInterface() = default;
 
@@ -335,6 +342,45 @@ class CORE_EXPORT QgsMeshDatasetSourceInterface SIP_ABSTRACT
                                       const QVector<QgsMeshDataBlock> &datasetActive,
                                       const QVector<double> &times
                                     ) = 0;
+
+
+    /**
+     * Saves a an existing dataset group provided by \a source to a file with a specified driver
+     *
+     * On success, the mesh's dataset group count is changed
+     *
+     * \param outputFilePath destination path of the stored file
+     * \param outputDriver output driver name
+     * \param source source of the dataset group
+     * \param datasetGroupIndex index of the dataset group in the \a source
+     *
+     * \returns TRUE on failure, FALSE on success
+     *
+     * \since QGIS 3.16
+     */
+    virtual bool persistDatasetGroup( const QString &outputFilePath,
+                                      const QString &outputDriver,
+                                      QgsMeshDatasetSourceInterface *source,
+                                      int datasetGroupIndex
+                                    ) = 0;
+
+    /**
+     * Returns the dataset index of the dataset in a specific dataet group at \a time from the \a reference time
+     *
+     * \param referenceTime the reference time from where to find the dataset
+     * \param groupIndex the index of the dataset group
+     * \param time the relative time from reference time
+     * \param method the method used to check the time
+     *
+     * \return the dataset index
+     */
+    QgsMeshDatasetIndex datasetIndexAtTime( const QDateTime &referenceTime,
+                                            int groupIndex,
+                                            quint64 time,
+                                            QgsMeshDataProviderTemporalCapabilities::MatchingTemporalDatasetMethod method ) const;
+
+  protected:
+    std::unique_ptr<QgsMeshDataProviderTemporalCapabilities> mTemporalCapabilities;
 };
 
 
@@ -351,10 +397,11 @@ class CORE_EXPORT QgsMeshDatasetSourceInterface SIP_ABSTRACT
 class CORE_EXPORT QgsMeshDataProvider: public QgsDataProvider, public QgsMeshDataSourceInterface, public QgsMeshDatasetSourceInterface
 {
     Q_OBJECT
-
   public:
     //! Ctor
-    QgsMeshDataProvider( const QString &uri, const QgsDataProvider::ProviderOptions &providerOptions );
+    QgsMeshDataProvider( const QString &uri,
+                         const QgsDataProvider::ProviderOptions &providerOptions,
+                         QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() );
 
     QgsMeshDataProviderTemporalCapabilities *temporalCapabilities() override;
     const QgsMeshDataProviderTemporalCapabilities *temporalCapabilities() const override SIP_SKIP;
@@ -372,8 +419,6 @@ class CORE_EXPORT QgsMeshDataProvider: public QgsDataProvider, public QgsMeshDat
     //! Emitted when some new dataset groups have been added
     void datasetGroupsAdded( int count );
 
-  private:
-    std::unique_ptr<QgsMeshDataProviderTemporalCapabilities> mTemporalCapabilities;
 };
 
 #endif // QGSMESHDATAPROVIDER_H
