@@ -29,7 +29,8 @@
 #include "qgspoint.h"
 #include "qgsdataprovider.h"
 
-
+class QgsMeshLayer;
+class QgsMeshDatasetGroup;
 class QgsRectangle;
 
 /**
@@ -174,7 +175,7 @@ class CORE_EXPORT QgsMeshDataBlock
      *
      * If the data provider/datasets does not have active
      * flag capability (== all values are valid), just
-     * set block validity by setValid( true )
+     * set block validity by setValid( TRUE )
      *
      * \param vals value vector with size count()
      *
@@ -375,6 +376,7 @@ class CORE_EXPORT QgsMeshDatasetGroupMetadata
      * \param extraOptions dataset's extra options stored by the provider. Usually contains the name, time value, time units, data file vendor, ...
      */
     QgsMeshDatasetGroupMetadata( const QString &name,
+                                 const QString uri,
                                  bool isScalar,
                                  DataType dataType,
                                  double minimum,
@@ -389,6 +391,13 @@ class CORE_EXPORT QgsMeshDatasetGroupMetadata
      */
     QString name() const;
 
+    /**
+     * Returns the uri of the source
+     *
+     * \since QGIS 3.16
+     */
+    QString uri() const;
+
     /**
      * Returns extra metadata options, for example description
      */
@@ -404,13 +413,11 @@ class CORE_EXPORT QgsMeshDatasetGroupMetadata
      */
     bool isScalar() const;
 
-
     /**
      * \brief Returns whether the dataset group is temporal (contains time-related dataset)
      */
     bool isTemporal() const;
 
-
     /**
      * Returns whether dataset group data is defined on vertices or faces or volumes
      *
@@ -444,6 +451,7 @@ class CORE_EXPORT QgsMeshDatasetGroupMetadata
 
   private:
     QString mName;
+    QString mUri;
     bool mIsScalar = false;
     DataType mDataType = DataType::DataOnFaces;
     double mMinimumValue = std::numeric_limits<double>::quiet_NaN();
@@ -521,6 +529,234 @@ class CORE_EXPORT QgsMeshDatasetMetadata
     int mMaximumVerticalLevelsCount = 0; // for 3d stacked meshes
 };
 
+
+/**
+ * \ingroup core
+ *
+ * Abstract class that represents a dataset
+ *
+ * \since QGIS 3.16
+ */
+class CORE_EXPORT QgsMeshDataset
+{
+  public:
+    //! Constructor
+    QgsMeshDataset() = default;
+
+    //! Destructor
+    virtual ~QgsMeshDataset() = default;
+
+    //! Returns the value with index \a valueIndex
+    virtual QgsMeshDatasetValue datasetValue( int valueIndex ) const = 0;
+
+    //! Returns \a count values from \a valueIndex
+    virtual QgsMeshDataBlock datasetValues( bool isScalar, int valueIndex, int count ) const = 0;
+
+    //! Returns whether faces are active
+    virtual QgsMeshDataBlock areFacesActive( int faceIndex, int count ) const = 0;
+
+    //! Returns whether the face is active
+    virtual bool isActive( int faceIndex ) const = 0;
+
+    //! Returns the metadata of the dataset
+    virtual QgsMeshDatasetMetadata metadata() const = 0;
+
+    //! Returns the values count
+    virtual int valuesCount() const = 0;
+};
+
+/**
+ * \ingroup core
+ *
+ * Abstract class that represents a dataset group
+ *
+ * \since QGIS 3.16
+ */
+class CORE_EXPORT QgsMeshDatasetGroup
+{
+  public:
+
+    /**
+     * Type of the dataset group
+     *
+     * \since QGIS 3.16
+     */
+    enum Type
+    {
+      None, //! Generic type used for non typed dataset group
+      Persistent, //! Dataset group store in a file
+      Memory, //! Temporary dataset group in memory
+      Virtual, //! Virtual Dataset group defined by a formula
+    };
+
+    //! Default constructor
+    QgsMeshDatasetGroup() = default;
+    virtual ~QgsMeshDatasetGroup();
+
+    //! Constructor with the \a name of the dataset group
+    QgsMeshDatasetGroup( const QString &name );
+
+    //! Constructor with the \a name of the dataset group and the \a dataTYpe
+    QgsMeshDatasetGroup( const QString &name, QgsMeshDatasetGroupMetadata::DataType dataType );
+
+    //! Initialize the dataset group
+    virtual void initialize() = 0;
+
+    //! Returns the metadata of the dataset group
+    QgsMeshDatasetGroupMetadata groupMetadata() const;
+
+    //! Returns the metadata of the dataset with index \a datasetIndex
+    virtual QgsMeshDatasetMetadata datasetMetadata( int datasetIndex ) const = 0 ;
+
+    //! Returns the count of datasets in the group
+    virtual int datasetCount() const = 0;
+
+    //! Returns the dataset with \a index
+    virtual QgsMeshDataset *dataset( int index ) const = 0;
+
+    //! Returns the type of dataset group
+    virtual QgsMeshDatasetGroup::Type type() const = 0;
+
+    //! Returns the minimum value of the whole dataset group
+    double minimum() const;
+
+    //! Returns the maximum value of the whole dataset group
+    double maximum() const;
+
+    //! Overrides the minimum and the maximum value of the whole dataset group
+    void setMinimumMaximum( double min, double max );
+
+    //! Returns the name of the dataset group
+    QString name() const;
+
+    //! Sets the name of the dataset group
+    void setName( const QString &name );
+
+    //! Returns the data type of the dataset group
+    QgsMeshDatasetGroupMetadata::DataType dataType() const;
+
+    //! Sets the data type of the dataset group
+    void setDataType( const QgsMeshDatasetGroupMetadata::DataType &dataType );
+
+    //! Adds extra metadata to the group
+    void addExtraMetadata( QString key, QString value );
+    //! Returns all the extra metadata of the group
+    QMap<QString, QString> extraMetadata() const;
+
+    //! Returns whether the group contain scalar values
+    bool isScalar() const;
+
+    //! Sets whether the group contain scalar values
+    void setIsScalar( bool isScalar );
+
+    //! Returns whether all the datasets contain \a count values
+    bool checkValueCountPerDataset( int count ) const;
+
+    //! Calculates the statictics (minimum and maximum)
+    void calculateStatistic();
+
+    //! Returns the dataset group variable name which this dataset group depends on
+    virtual QStringList datasetGroupNamesDependentOn() const;
+
+    //! Write dataset group information in a DOM element
+    virtual QDomElement writeXml( QDomDocument &doc, const QgsReadWriteContext &context ) const = 0;
+
+    //! Returns some information about the dataset group
+    virtual QString description() const;
+
+    //! Sets the reference time of the dataset group
+    void setReferenceTime( const QDateTime &referenceTime );
+
+  protected:
+    QString mName;
+
+    QgsMeshDatasetGroupMetadata::DataType mDataType = QgsMeshDatasetGroupMetadata::DataOnVertices;
+    QMap<QString, QString> mMetadata;
+    bool mIsScalar = true;
+
+  private:
+    double mMinimum = std::numeric_limits<double>::quiet_NaN();
+    double mMaximum = std::numeric_limits<double>::quiet_NaN();
+
+    QDateTime mReferenceTime;
+};
+
+#ifndef SIP_RUN
+
+/**
+ * \ingroup core
+ *
+ * Class to store memory dataset
+ * The QgsMeshDatasetValue objects and whether the faces are active are stored in QVector containers that are exposed for efficiency
+ *
+ * \since QGIS 3.16
+ */
+class CORE_EXPORT QgsMeshMemoryDataset: public QgsMeshDataset
+{
+  public:
+    //! Constructor
+    QgsMeshMemoryDataset() = default;
+
+    QgsMeshDatasetValue datasetValue( int valueIndex ) const override;
+    QgsMeshDataBlock datasetValues( bool isScalar, int valueIndex, int count ) const override;
+    QgsMeshDataBlock areFacesActive( int faceIndex, int count ) const override;
+    QgsMeshDatasetMetadata metadata() const override;
+    bool isActive( int faceIndex ) const override;
+    int valuesCount() const override;
+
+    //! Calculates the minimum and the maximum of this group
+    void calculateMinMax();
+
+    QVector<QgsMeshDatasetValue> values;
+    QVector<int> active;
+    double time = -1;
+    bool valid = false;
+    double minimum = std::numeric_limits<double>::quiet_NaN();
+    double maximum = std::numeric_limits<double>::quiet_NaN();
+};
+
+/**
+ * \ingroup core
+ *
+ * Class that represents a dataset group stored in memory
+ * The QgsMeshMemoryDataset objects stores in a QVector container that are exposed for efficiency
+ *
+ * \since QGIS 3.16
+ */
+class CORE_EXPORT QgsMeshMemoryDatasetGroup: public QgsMeshDatasetGroup
+{
+  public:
+    //! Constructor
+    QgsMeshMemoryDatasetGroup() = default;
+    //! Constructor with the \a name of the group
+    QgsMeshMemoryDatasetGroup( const QString &name );
+    //! Constructor with the \a name of the group and the type of data \a dataType
+    QgsMeshMemoryDatasetGroup( const QString &name, QgsMeshDatasetGroupMetadata::DataType dataType );
+
+    void initialize() override;
+    int datasetCount() const override;
+    QgsMeshDatasetMetadata datasetMetadata( int datasetIndex ) const override;
+    QgsMeshDataset *dataset( int index ) const override;
+    virtual QgsMeshDatasetGroup::Type type() const override {return QgsMeshDatasetGroup::Memory;}
+
+    //! Returns a invalid DOM element
+    QDomElement writeXml( QDomDocument &doc, const QgsReadWriteContext &context )  const override;
+
+    //! Adds a memory dataset to the group
+    void addDataset( std::shared_ptr<QgsMeshMemoryDataset> dataset );
+
+    //! Removes all the datasets from the group
+    void clearDatasets();
+
+    //! Returns the dataset with \a index
+    std::shared_ptr<const QgsMeshMemoryDataset> constDataset( int index ) const;
+
+    //! Contains all the memory datasets
+    QVector<std::shared_ptr<QgsMeshMemoryDataset>> memoryDatasets;
+};
+
+#endif //SIP_RUN
+
 /**
  * \ingroup core
  *
@@ -540,6 +776,8 @@ class CORE_EXPORT QgsMeshDatasetMetadata
  *     Minimum
  *     Maximum
  *
+ * Tree items handle also the dependencies between dataset groups represented by these items
+ *
  * \since QGIS 3.14 in core API
  */
 
@@ -556,15 +794,17 @@ class CORE_EXPORT QgsMeshDatasetGroupTreeItem
      * Constructor
      *
      * \param defaultName the name that will be used to display the item if iot not overrides (\see setName())
+     * \param sourceName the name used by the source (provider, dataset group store,...)
      * \param isVector whether the dataset group is a vector dataset group
      * \param index index of the dataset group
      */
     QgsMeshDatasetGroupTreeItem( const QString &defaultName,
+                                 const QString &sourceName,
                                  bool isVector,
                                  int index );
 
     /**
-     * Constructor from a DOM element, contruct also the children
+     * Constructor from a DOM element, constructs also the children
      *
      * \param itemElement the DOM element
      * \param context writing context (e.g. for conversion between relative and absolute paths)
@@ -572,7 +812,7 @@ class CORE_EXPORT QgsMeshDatasetGroupTreeItem
     QgsMeshDatasetGroupTreeItem( const QDomElement &itemElement, const QgsReadWriteContext &context );
 
     /**
-     * Destructor, destructs also the chilren
+     * Destructor, destructs also the children
      *
     */
     ~QgsMeshDatasetGroupTreeItem();
@@ -592,10 +832,20 @@ class CORE_EXPORT QgsMeshDatasetGroupTreeItem
      */
     void appendChild( QgsMeshDatasetGroupTreeItem *item SIP_TRANSFER );
 
+    /**
+     * Removes a item child if exists
+     * \param item the item to append
+     *
+     * \note takes ownership of item
+     *
+     * \since QGIS 3.16
+     */
+    void removeChild( QgsMeshDatasetGroupTreeItem *item SIP_TRANSFER );
+
     /**
      * Returns a child
      * \param row the position of the child
-     * \return the item at the positon \a row
+     * \return the item at the position \a row
      */
     QgsMeshDatasetGroupTreeItem *child( int row ) const;
 
@@ -615,11 +865,19 @@ class CORE_EXPORT QgsMeshDatasetGroupTreeItem
     int childCount() const;
 
     /**
-     * Returns the total count of children, that is included deeper children
-     * \return
-     */
+    * Returns the total count of children, that is included deeper children and disabled items
+    * \return the total children's count
+    */
     int totalChildCount() const;
 
+    /**
+     * Returns a list of enabled dataset group indexes, included deeper children
+     * \return the list of dataset group indexes
+     *
+     * \since QGIS 3.16.3
+     */
+    QList<int> enabledDatasetGroupIndexes() const;
+
     /**
      * Returns the parent item, nullptr if it is root item
      * \return the parent item
@@ -634,7 +892,7 @@ class CORE_EXPORT QgsMeshDatasetGroupTreeItem
 
     /**
      * Returns the name of the item
-     * This mame is the default name if the name has not been overrided (\see setName())
+     * This name is the default name if the name has not been overridden (\see setName())
      * \return the name to display
      */
     QString name() const;
@@ -642,11 +900,20 @@ class CORE_EXPORT QgsMeshDatasetGroupTreeItem
     /**
      * Overrides the default name with the name to display.
      * The default name is still stored in the item
-     * but will not be displayed anymore except if the empty string is setted.
+     * but will not be displayed anymore except if the empty string is set.
      * \param name to display
      */
     void setName( const QString &name );
 
+    /**
+     * Returns the name used by the provider to identify the dataset
+     *
+     * \return the provider name
+     *
+     * \since QGIS 3.16
+     */
+    QString providerName() const;
+
     /**
      * \return whether the dataset group is vector
      */
@@ -674,7 +941,47 @@ class CORE_EXPORT QgsMeshDatasetGroupTreeItem
     QString defaultName() const;
 
     /**
-     * Write the item and its children in a DOM document
+     * \return the dataset group type
+     *
+     * \since QGIS 3.16
+     */
+    QgsMeshDatasetGroup::Type datasetGroupType() const;
+
+    /**
+     * Returns a list of group index corresponding to dataset group that depends on the dataset group represented by this item
+     *
+     * \return list of group index
+     *
+     */
+    QList<int> groupIndexDependencies() const;
+
+    /**
+     * Returns description about the dataset group (URI, formula,...)
+     *
+     * \since QGIS 3.16
+     */
+    QString description() const;
+
+    /**
+     * Set parameters of the item in accordance with the dataset group
+     *
+     * \param datasetGroup pointer to the dataset group to accord with
+     *
+     * \since QGIS 3.16
+     */
+    void setDatasetGroup( QgsMeshDatasetGroup *datasetGroup );
+
+    /**
+     * Set parameters of the item in accordance with the persistent dataset group with \a uri
+     *
+     * \param uri uri of the persistent dataset group
+     *
+     * \since QGIS 3.16
+     */
+    void setPersistentDatasetGroup( const QString &uri );
+
+    /**
+     * Writes the item and its children in a DOM document
      * \param doc the DOM document
      * \param context writing context (e.g. for conversion between relative and absolute paths)
      * \return the dom element where the item is written
@@ -688,10 +995,22 @@ class CORE_EXPORT QgsMeshDatasetGroupTreeItem
 
     // Data
     QString mUserName;
-    QString mProviderName;
+    QString mOriginalName;
+    QString mSourceName;
+    QgsMeshDatasetGroup::Type mDatasetGroupType = QgsMeshDatasetGroup::None;
+    QString mDescription;
+
     bool mIsVector = false;
     int mDatasetGroupIndex = -1;
     bool mIsEnabled = true;
+
+    QList<int> mDatasetGroupDependencies;
+    QList<int> mDatasetGroupDependentOn;
+
+    QgsMeshDatasetGroupTreeItem *searchItemBySourceName( const QString &sourceName ) const;
+    QgsMeshDatasetGroupTreeItem *rootItem() const;
+    void freeAsDependency();
+    void freeFromDependencies();
 };
 
 #endif // QGSMESHDATASET_H
