@@ -36,6 +36,7 @@ class QgsRenderContext;
 struct QgsMesh;
 class QgsMesh3dAveragingMethod;
 class QgsMeshLayerTemporalProperties;
+class QgsMeshDatasetGroupStore;
 
 /**
  * \ingroup core
@@ -63,9 +64,8 @@ class QgsMeshLayerTemporalProperties;
  * vertices and faces is comma separated coordinates and connections for mesh.
  * E.g. to create mesh with one quad and one triangle
  *
- * \code
- *  QString uri(
- *      "1.0, 2.0 \n" \
+ * \code{py}
+ *  uri = "1.0, 2.0 \n" \
  *      "2.0, 2.0 \n" \
  *      "3.0, 2.0 \n" \
  *      "2.0, 3.0 \n" \
@@ -73,8 +73,8 @@ class QgsMeshLayerTemporalProperties;
  *      "---" \
  *      "0, 1, 3, 4 \n" \
  *      "1, 2, 3 \n"
- *    );
- *    QgsMeshLayer *scratchLayer = new QgsMeshLayer(uri, "My Scratch layer", "mesh_memory");
+ *
+ *  scratchLayer = QgsMeshLayer(uri, "My Scratch layer", "mesh_memory")
  * \endcode
  *
  * \subsection mdal MDAL data provider (mdal)
@@ -82,9 +82,9 @@ class QgsMeshLayerTemporalProperties;
  * Accesses data using the MDAL drivers (https://github.com/lutraconsulting/MDAL). The url
  * is the MDAL connection string. QGIS must be built with MDAL support to allow this provider.
 
- * \code
- *     QString uri = "test/land.2dm";
- *     QgsMeshLayer *scratchLayer = new QgsMeshLayer(uri, "My Scratch Layer",  "mdal");
+ * \code{py}
+ *     uri = "test/land.2dm"
+ *     scratchLayer = QgsMeshLayer(uri, "My Scratch Layer",  "mdal")
  * \endcode
  *
  * \note The API is considered EXPERIMENTAL and can be changed without a notice
@@ -151,6 +151,14 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
     //! QgsMeshLayer cannot be copied.
     QgsMeshLayer &operator=( QgsMeshLayer const &rhs ) = delete;
 
+#ifdef SIP_RUN
+    SIP_PYOBJECT __repr__();
+    % MethodCode
+    QString str = QStringLiteral( "<QgsMeshLayer: '%1' (%2)>" ).arg( sipCpp->name(), sipCpp->dataProvider() ? sipCpp->dataProvider()->name() : QStringLiteral( "Invalid" ) );
+    sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+    % End
+#endif
+
     QgsMeshDataProvider *dataProvider() override;
     const QgsMeshDataProvider *dataProvider() const override SIP_SKIP;
     QgsMeshLayer *clone() const override SIP_FACTORY;
@@ -169,12 +177,13 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
     QgsMapLayerTemporalProperties *temporalProperties() override;
     void reload() override;
     QStringList subLayers() const override;
+    QString htmlMetadata() const override;
 
     //! Returns the provider type for this layer
     QString providerType() const;
 
     /**
-     * Add datasets to the mesh from file with \a path. Use the the time \a defaultReferenceTime as reference time is not provided in the file
+     * Adds datasets to the mesh from file with \a path. Use the the time \a defaultReferenceTime as reference time is not provided in the file
      *
      * \param path the path to the atasets file
      * \param defaultReferenceTime reference time used if not provided in the file
@@ -184,6 +193,28 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
      */
     bool addDatasets( const QString &path, const QDateTime &defaultReferenceTime = QDateTime() );
 
+    /**
+     * Adds extra datasets to the mesh. Take ownership.
+     *
+     * \param datasetGroup the extra dataset group
+     * \return whether the dataset is effectively added
+     *
+     * \since QGIS 3.16
+     */
+    bool addDatasets( QgsMeshDatasetGroup *datasetGroup SIP_TRANSFER );
+
+    /**
+     * Saves datasets group on file with the specified \a driver
+     *
+     * \param path the path of the file
+     * \param datasetGroupIndex the index of the dataset group
+     * \param driver the driver to used for saving
+     * \return false if succeeds
+     *
+     * \since QGIS 3.16
+     */
+    bool saveDataset( const QString &path, int datasetGroupIndex, QString driver );
+
     /**
      * Returns native mesh (NULLPTR before rendering or calling to updateMesh)
      *
@@ -269,6 +300,154 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
      */
     QString formatTime( double hours );
 
+    /**
+     * Returns the dataset groups count handle by the layer
+     *
+     * \since QGIS 3.16
+     */
+    int datasetGroupCount() const;
+
+    /**
+     * Returns the extra dataset groups count handle by the layer
+     *
+     * \since QGIS 3.16
+     */
+    int extraDatasetGroupCount() const;
+
+    /**
+     * Returns the list of indexes of dataset groups count handled by the layer
+     *
+     * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+     * In the layer scope, those indexes can be different from the data provider indexes.
+     *
+     * \since QGIS 3.16
+     */
+    QList<int> datasetGroupsIndexes() const;
+
+    /**
+     * Returns the list of indexes of enables dataset groups handled by the layer
+     *
+     * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+     * In the layer scope, those indexes can be different from the data provider indexes.
+     *
+     * \since QGIS 3.16.3
+     */
+    QList<int> enabledDatasetGroupsIndexes() const;
+
+    /**
+     * Returns the dataset groups metadata
+     *
+     * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+     * In the layer scope, those indexes can be different from the data provider indexes.
+     *
+     * \since QGIS 3.16
+     */
+    QgsMeshDatasetGroupMetadata datasetGroupMetadata( const QgsMeshDatasetIndex &index ) const;
+
+    /**
+     * Returns the dataset count in the dataset groups
+     *
+     * \param index index of the dataset in the group
+     *
+     * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+     * In the layer scope, those indexes can be different from the data provider indexes.
+     *
+     * \since QGIS 3.16
+     */
+    int datasetCount( const QgsMeshDatasetIndex &index ) const;
+
+    /**
+     * Returns the dataset metadata
+     *
+     * \param index index of the dataset
+     *
+     * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+     * In the layer scope, those indexes can be different from the data provider indexes.
+     *
+     * \since QGIS 3.16
+     */
+    QgsMeshDatasetMetadata datasetMetadata( const QgsMeshDatasetIndex &index ) const;
+
+    /**
+     * Returns  vector/scalar value associated with the index from the dataset
+     * To read multiple continuous values, use datasetValues()
+     *
+     * See QgsMeshDatasetMetadata::isVector() or QgsMeshDataBlock::type()
+     * to check if the returned value is vector or scalar
+     *
+     * Returns invalid value for DataOnVolumes
+     *
+     * \param index index of the dataset
+     * \param valueIndex index of the value
+     *
+     * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+     * In the layer scope, those indexes can be different from the data provider indexes.
+     *
+     * \since QGIS 3.16
+     */
+    QgsMeshDatasetValue datasetValue( const QgsMeshDatasetIndex &index, int valueIndex ) const;
+
+    /**
+     * Returns N vector/scalar values from the index from the dataset
+     *
+     * See QgsMeshDatasetMetadata::isVector() or QgsMeshDataBlock::type()
+     * to check if the returned value is vector or scalar
+     *
+     * Returns invalid block for DataOnVolumes. Use QgsMeshLayerUtils::datasetValues() if you
+     * need block for any type of data type
+     *
+     * \param index index of the dataset
+     * \param valueIndex index of the value
+     * \param count number of values to return
+     *
+     * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+     * In the layer scope, those indexes can be different from the data provider indexes.
+     *
+     * \since QGIS 3.16
+     */
+    QgsMeshDataBlock datasetValues( const QgsMeshDatasetIndex &index, int valueIndex, int count ) const;
+
+    /**
+     * Returns N vector/scalar values from the face index from the dataset for 3d stacked meshes
+     *
+     * See QgsMeshDatasetMetadata::isVector() to check if the returned value is vector or scalar
+     *
+     * returns invalid block for DataOnFaces and DataOnVertices.
+     *
+     * \param index index of the dataset
+     * \param valueIndex index of the value
+     * \param count number of values to return
+     *
+     * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+     * In the layer scope, those indexes can be different from the data provider indexes.
+     *
+     * \since QGIS 3.16
+     */
+    QgsMesh3dDataBlock dataset3dValues( const QgsMeshDatasetIndex &index, int faceIndex, int count ) const;
+
+    /**
+     * Returns N vector/scalar values from the face index from the dataset for 3d stacked meshes
+     *
+     * See QgsMeshDatasetMetadata::isVector() to check if the returned value is vector or scalar
+     *
+     * returns invalid block for DataOnFaces and DataOnVertices.
+     */
+    bool isFaceActive( const QgsMeshDatasetIndex &index, int faceIndex ) const;
+
+    /**
+     * Returns whether the faces are active for particular dataset
+     *
+     * \param index index of the dataset
+     * \param valueIndex index of the value
+     * \param count number of values to return
+     *
+     * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+     * In the layer scope, those indexes are different from the data provider indexes.
+     *
+     * \since QGIS 3.16
+     */
+    QgsMeshDataBlock areFacesActive( const QgsMeshDatasetIndex &index, int faceIndex, int count ) const;
+
     /**
       * Interpolates the value on the given point from given dataset.
       * For 3D datasets, it uses dataset3dValue(), \n
@@ -286,6 +465,9 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
       * outside the mesh layer, nodata values and in case triangular mesh was not
       * previously used for rendering
       *
+      * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+      * In the layer scope, those indexes are different from the data provider indexes.
+      *
       * \since QGIS 3.4
       */
     QgsMeshDatasetValue datasetValue( const QgsMeshDatasetIndex &index, const QgsPointXY &point, double searchRadius = 0 ) const;
@@ -304,6 +486,9 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
       * for point outside the mesh layer or in case triangular mesh was not
       * previously used for rendering or for datasets that do not have type DataOnVolumes
       *
+      * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+      * In the layer scope, those indexes are different from the data provider indexes.
+      *
       * \since QGIS 3.12
       */
     QgsMesh3dDataBlock dataset3dValue( const QgsMeshDatasetIndex &index, const QgsPointXY &point ) const;
@@ -322,24 +507,45 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
       * \returns interpolated value at the projected point. Returns NaN values for values
       * outside the mesh layer and in case triangular mesh was not previously used for rendering
       *
+      * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+      * In the layer scope, those indexes are different from the data provider indexes.
+      *
       * \since QGIS 3.14
       */
     QgsMeshDatasetValue dataset1dValue( const QgsMeshDatasetIndex &index, const QgsPointXY &point, double searchRadius ) const;
 
-
     /**
       * Returns dataset index from datasets group depending on the time range.
-      * If the temporal properties is not active, returns invalid dataset index
+      * If the temporal properties is not active, returns invalid dataset index. This method is used for rendering mesh layer.
       *
       * \param timeRange the time range
       * \returns dataset index
       *
       * \note the returned dataset index depends on the matching method, see setTemporalMatchingMethod()
       *
+      * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+      * In the layer scope, those indexes are different from the data provider indexes.
+      *
       * \since QGIS 3.14
       */
     QgsMeshDatasetIndex datasetIndexAtTime( const QgsDateTimeRange &timeRange, int datasetGroupIndex ) const;
 
+    /**
+      * Returns dataset index from datasets group depending on the relative time from the layer reference time.
+      * Dataset index is valid even the temporal properties is inactive. This method is used for calculation on mesh layer.
+      *
+      * \param relativeTime the relative from the mesh layer reference time
+      * \returns dataset index
+      *
+      * \note the returned dataset index depends on the matching method, see setTemporalMatchingMethod()
+      *
+      * \note indexes are used to distinguish all the dataset groups handled by the layer (from dataprovider, extra dataset group,...)
+      * In the layer scope, those indexes are different from the data provider indexes.
+      *
+      * \since QGIS 3.16
+      */
+    QgsMeshDatasetIndex datasetIndexAtRelativeTime( const QgsInterval &relativeTime, int datasetGroupIndex ) const;
+
     /**
       * Returns dataset index from active scalar group depending on the time range.
       * If the temporal properties is not active, return the static dataset
@@ -477,6 +683,20 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
      */
     QgsInterval firstValidTimeStep() const;
 
+    /**
+     * Returns the relative time of the dataset from the reference time of its group
+     *
+     * \since QGIS 3.16
+     */
+    QgsInterval datasetRelativeTime( const QgsMeshDatasetIndex &index );
+
+    /**
+     * Returns the relative time (in milliseconds) of the dataset from the reference time of its group
+     *
+     * \since QGIS 3.16
+     */
+    qint64 datasetRelativeTimeInMilliseconds( const QgsMeshDatasetIndex &index );
+
   public slots:
 
     /**
@@ -520,17 +740,17 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
      * Binds layer to a specific data provider
      * \param provider provider key string, must match a valid QgsMeshDataProvider key. E.g. "mesh_memory", etc.
      * \param options generic provider options
+     * \param flags provider flags since QGIS 3.16
      */
-    bool setDataProvider( QString const &provider, const QgsDataProvider::ProviderOptions &options );
+    bool setDataProvider( QString const &provider, const QgsDataProvider::ProviderOptions &options, QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() );
 
 #ifdef SIP_RUN
     QgsMeshLayer( const QgsMeshLayer &rhs );
 #endif
 
-  private:
     void fillNativeMesh();
     void assignDefaultStyleToDatasetGroup( int groupIndex );
-    void setDefaultRendererSettings();
+    void setDefaultRendererSettings( const QList<int> &groupIndexes );
     void createSimplifiedMeshes();
     int levelsOfDetailsIndex( double partOfMeshInView ) const;
 
@@ -540,12 +760,14 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
     void applyClassificationOnScalarSettings( const QgsMeshDatasetGroupMetadata &meta, QgsMeshRendererScalarSettings &scalarSettings ) const;
 
   private slots:
-    void onDatasetGroupsAdded( int count );
+    void onDatasetGroupsAdded( const QList<int> &datasetGroupIndexes );
 
   private:
     //! Pointer to data provider derived from the abastract base class QgsMeshDataProvider
     QgsMeshDataProvider *mDataProvider = nullptr;
 
+    std::unique_ptr<QgsMeshDatasetGroupStore> mDatasetGroupStore;
+
     //! Pointer to native mesh structure, used as cache for rendering
     std::unique_ptr<QgsMesh> mNativeMesh;
 
@@ -569,8 +791,6 @@ class CORE_EXPORT QgsMeshLayer : public QgsMapLayer
     int mStaticScalarDatasetIndex = 0;
     int mStaticVectorDatasetIndex = 0;
 
-    std::unique_ptr<QgsMeshDatasetGroupTreeItem> mDatasetGroupTreeRootItem;
-
     int closestEdge( const QgsPointXY &point, double searchRadius, QgsPointXY &projectedPoint ) const;
 
     //! Returns the exact position in map coordinates of the closest vertex in the search area
