@@ -53,7 +53,7 @@ class CORE_EXPORT QgsRasterBlock
      *  \param dataType raster data type
      *  \param width width of data matrix
      *  \param height height of data matrix
-     *  \returns true on success
+     *  \returns TRUE on success
      */
     bool reset( Qgis::DataType dataType, int width, int height );
 
@@ -61,24 +61,26 @@ class CORE_EXPORT QgsRasterBlock
     // and works also if block is valid but empty - difference between valid and empty?
 
     /**
-     * \brief Returns true if the block is valid (correctly filled with data).
+     * \brief Returns TRUE if the block is valid (correctly filled with data).
      *  An empty block may still be valid (if zero size block was requested).
      *  If the block is not valid, error may be retrieved by error() method.
      */
-    bool isValid() const { return mValid; }
+    bool isValid() const SIP_HOLDGIL { return mValid; }
 
     //! \brief Mark block as valid or invalid
-    void setValid( bool valid ) { mValid = valid; }
+    void setValid( bool valid ) SIP_HOLDGIL { mValid = valid; }
 
     /**
-     * Returns true if block is empty, i.e. its size is 0 (zero rows or cols).
-     *  This method does not return true if size is not zero and all values are
+     * Returns TRUE if block is empty, i.e. its size is 0 (zero rows or cols).
+     *  This method does not return TRUE if size is not zero and all values are
      *  'no data' (null).
      */
     bool isEmpty() const;
 
-    // Return data type size in bytes
-    static int typeSize( int dataType )
+    /**
+     * Returns the size in bytes for the specified \a dataType.
+     */
+    static int typeSize( int dataType ) SIP_HOLDGIL
     {
       // Modified and extended copy from GDAL
       switch ( dataType )
@@ -113,37 +115,40 @@ class CORE_EXPORT QgsRasterBlock
       }
     }
 
-    // Data type in bytes
-    int dataTypeSize() const
+    /**
+     * Data type size in bytes.
+     */
+    int dataTypeSize() const SIP_HOLDGIL
     {
       return typeSize( mDataType );
     }
 
-    //! Returns true if data type is numeric
+    //! Returns TRUE if data type is numeric
     static bool typeIsNumeric( Qgis::DataType type );
 
-    //! Returns true if data type is color
+    //! Returns TRUE if data type is color
     static bool typeIsColor( Qgis::DataType type );
 
     //! Returns data type
-    Qgis::DataType dataType() const { return mDataType; }
+    Qgis::DataType dataType() const  SIP_HOLDGIL { return mDataType; }
 
     //! For given data type returns wider type and sets no data value
     static Qgis::DataType typeWithNoDataValue( Qgis::DataType dataType, double *noDataValue );
 
     /**
-     * True if the block has no data value.
-     * \returns true if the block has no data value
+     * TRUE if the block has no data value.
+     * \returns TRUE if the block has no data value
      * \see noDataValue(), setNoDataValue(), resetNoDataValue()
      */
-    bool hasNoDataValue() const { return mHasNoDataValue; }
+    bool hasNoDataValue() const SIP_HOLDGIL { return mHasNoDataValue; }
 
     /**
-     * Returns true if the block may contain no data. It does not guarantee
+     * Returns TRUE if the block may contain no data. It does not guarantee
      * that it really contains any no data. It can be used to speed up processing.
      * Not the difference between this method and hasNoDataValue().
-     * \returns true if the block may contain no data */
-    bool hasNoData() const
+     * \returns TRUE if the block may contain no data
+    */
+    bool hasNoData() const SIP_HOLDGIL
     {
       return mHasNoDataValue || mNoDataBitmap;
     }
@@ -153,7 +158,7 @@ class CORE_EXPORT QgsRasterBlock
      * \see noDataValue(), hasNoDataValue(), resetNoDataValue()
      * \since QGIS 3.0
      */
-    void setNoDataValue( double noDataValue );
+    void setNoDataValue( double noDataValue ) SIP_HOLDGIL;
 
     /**
      * Reset no data value: if there was a no data value previously set,
@@ -161,7 +166,7 @@ class CORE_EXPORT QgsRasterBlock
      * \see noDataValue(), hasNoDataValue(), setNoDataValue()
      * \since QGIS 3.0
      */
-    void resetNoDataValue();
+    void resetNoDataValue() SIP_HOLDGIL;
 
     /**
      * Returns no data value. If the block does not have a no data value the
@@ -169,13 +174,14 @@ class CORE_EXPORT QgsRasterBlock
      * \returns No data value
      * \see hasNoDataValue(), setNoDataValue(), resetNoDataValue()
      */
-    double noDataValue() const { return mNoDataValue; }
+    double noDataValue() const SIP_HOLDGIL { return mNoDataValue; }
 
     /**
      * Gets byte array representing a value.
      * \param dataType data type
      * \param value value
-     * \returns byte array representing the value */
+     * \returns byte array representing the value
+    */
     static QByteArray valueBytes( Qgis::DataType dataType, double value );
 
     /**
@@ -186,7 +192,7 @@ class CORE_EXPORT QgsRasterBlock
      * \returns value
      * \see valueAndNoData()
     */
-    double value( int row, int column ) const
+    double value( int row, int column ) const SIP_HOLDGIL
     {
       return value( static_cast< qgssize >( row ) * mWidth + column );
     }
@@ -195,7 +201,7 @@ class CORE_EXPORT QgsRasterBlock
      * Reads a single value from the pixel at \a row and \a column, if type of block is numeric. If type is color,
      * returned value is undefined.
      *
-     * Additionally, the \a isNoData argument will be set to true if the pixel represents a nodata value. This method
+     * Additionally, the \a isNoData argument will be set to TRUE if the pixel represents a nodata value. This method
      * is more efficient then calling isNoData() and value() separately.
      *
      * \note Not available in Python bindings
@@ -215,13 +221,13 @@ class CORE_EXPORT QgsRasterBlock
      * \returns value
      * \see valueAndNoData()
     */
-    double value( qgssize index ) const;
+    inline double value( qgssize index ) const SIP_HOLDGIL;
 
     /**
      * Reads a single value from the pixel at the specified data matrix \a index, if type of block is numeric. If type is color,
      * returned value is undefined.
      *
-     * Additionally, the \a isNoData argument will be set to true if the pixel represents a nodata value. This method
+     * Additionally, the \a isNoData argument will be set to TRUE if the pixel represents a nodata value. This method
      * is more efficient then calling isNoData() and value() separately.
      *
      * \note Not available in Python bindings
@@ -229,11 +235,11 @@ class CORE_EXPORT QgsRasterBlock
      * \see isNoData()
      * \since QGIS 3.6
      */
-    double valueAndNoData( qgssize index, bool &isNoData ) const SIP_SKIP;
+    inline double valueAndNoData( qgssize index, bool &isNoData ) const SIP_SKIP;
 
     /**
      * Gives direct access to the raster block data.
-     * The data type of the block must be Qgis::Byte otherwise it returns null pointer.
+     * The data type of the block must be Qgis::Byte otherwise it returns NULLPTR.
      * Useful for most efficient read access.
      * \note not available in Python bindings
      * \since QGIS 3.4
@@ -249,8 +255,9 @@ class CORE_EXPORT QgsRasterBlock
      * \brief Read a single color
      *  \param row row index
      *  \param column column index
-     *  \returns color */
-    QRgb color( int row, int column ) const
+     *  \returns color
+    */
+    QRgb color( int row, int column ) const SIP_HOLDGIL
     {
       if ( !mImage ) return NO_DATA_COLOR;
 
@@ -260,8 +267,9 @@ class CORE_EXPORT QgsRasterBlock
     /**
      * \brief Read a single value
      *  \param index data matrix index (long type in Python)
-     *  \returns color */
-    QRgb color( qgssize index ) const
+     *  \returns color
+    */
+    QRgb color( qgssize index ) const SIP_HOLDGIL
     {
       int row = static_cast< int >( std::floor( static_cast< double >( index ) / mWidth ) );
       int column = index % mWidth;
@@ -272,10 +280,10 @@ class CORE_EXPORT QgsRasterBlock
      * Checks if value at position is no data
      * \param row row index
      * \param column column index
-     * \returns true if value is no data
+     * \returns TRUE if value is no data
      * \see valueAndNoData()
     */
-    bool isNoData( int row, int column ) const
+    bool isNoData( int row, int column ) const SIP_HOLDGIL
     {
       return isNoData( static_cast< qgssize >( row ) * mWidth + column );
     }
@@ -284,10 +292,10 @@ class CORE_EXPORT QgsRasterBlock
      * Check if value at position is no data
      * \param row row index
      * \param column column index
-     * \returns true if value is no data
+     * \returns TRUE if value is no data
      * \see valueAndNoData()
     */
-    bool isNoData( qgssize row, qgssize column ) const
+    bool isNoData( qgssize row, qgssize column ) const SIP_HOLDGIL
     {
       return isNoData( row * static_cast< qgssize >( mWidth ) + column );
     }
@@ -295,10 +303,10 @@ class CORE_EXPORT QgsRasterBlock
     /**
      * Check if value at position is no data
      * \param index data matrix index (long type in Python)
-     * \returns true if value is no data
+     * \returns TRUE if value is no data
      * \see valueAndNoData()
     */
-    bool isNoData( qgssize index ) const
+    bool isNoData( qgssize index ) const SIP_HOLDGIL
     {
       if ( !mHasNoDataValue && !mNoDataBitmap )
         return false;
@@ -334,8 +342,9 @@ class CORE_EXPORT QgsRasterBlock
      *  \param row row index
      *  \param column column index
      *  \param value the value to be set
-     *  \returns true on success */
-    bool setValue( int row, int column, double value )
+     *  \returns TRUE on success
+    */
+    bool setValue( int row, int column, double value ) SIP_HOLDGIL
     {
       return setValue( static_cast< qgssize >( row ) * mWidth + column, value );
     }
@@ -344,8 +353,9 @@ class CORE_EXPORT QgsRasterBlock
      * \brief Set value on index (indexed line by line)
      *  \param index data matrix index (long type in Python)
      *  \param value the value to be set
-     *  \returns true on success */
-    bool setValue( qgssize index, double value )
+     *  \returns TRUE on success
+    */
+    bool setValue( qgssize index, double value ) SIP_HOLDGIL
     {
       if ( !mData )
       {
@@ -366,8 +376,9 @@ class CORE_EXPORT QgsRasterBlock
      *  \param row row index
      *  \param column column index
      *  \param color the color to be set, QRgb value
-     *  \returns true on success */
-    bool setColor( int row, int column, QRgb color )
+     *  \returns TRUE on success
+    */
+    bool setColor( int row, int column, QRgb color ) SIP_HOLDGIL
     {
       return setColor( static_cast< qgssize >( row ) * mWidth + column, color );
     }
@@ -376,8 +387,9 @@ class CORE_EXPORT QgsRasterBlock
      * \brief Set color on index (indexed line by line)
      *  \param index data matrix index (long type in Python)
      *  \param color the color to be set, QRgb value
-     *  \returns true on success */
-    bool setColor( qgssize index, QRgb color )
+     *  \returns TRUE on success
+    */
+    bool setColor( qgssize index, QRgb color ) SIP_HOLDGIL
     {
       if ( !mImage )
       {
@@ -399,7 +411,7 @@ class CORE_EXPORT QgsRasterBlock
 
     /**
      * Gives direct read/write access to the raster RGB data.
-     * The data type of the block must be Qgis::ARGB32 or Qgis::ARGB32_Premultiplied otherwise it returns null pointer.
+     * The data type of the block must be Qgis::ARGB32 or Qgis::ARGB32_Premultiplied otherwise it returns NULLPTR.
      * Useful for most efficient read/write access to RGB blocks.
      * \note not available in Python bindings
      * \since QGIS 3.4
@@ -415,8 +427,9 @@ class CORE_EXPORT QgsRasterBlock
      * \brief Set no data on pixel
      *  \param row row index
      *  \param column column index
-     *  \returns true on success */
-    bool setIsNoData( int row, int column )
+     *  \returns TRUE on success
+    */
+    bool setIsNoData( int row, int column ) SIP_HOLDGIL
     {
       return setIsNoData( static_cast< qgssize >( row ) * mWidth + column );
     }
@@ -424,8 +437,9 @@ class CORE_EXPORT QgsRasterBlock
     /**
      * \brief Set no data on pixel
      *  \param index data matrix index (long type in Python)
-     *  \returns true on success */
-    bool setIsNoData( qgssize index )
+     *  \returns TRUE on success
+    */
+    bool setIsNoData( qgssize index ) SIP_HOLDGIL
     {
       if ( mHasNoDataValue )
       {
@@ -454,12 +468,14 @@ class CORE_EXPORT QgsRasterBlock
 
     /**
      * \brief Set the whole block to no data
-     *  \returns true on success */
+     *  \returns TRUE on success
+    */
     bool setIsNoData();
 
     /**
      * \brief Set the whole block to no data except specified rectangle
-     *  \returns true on success */
+     *  \returns TRUE on success
+    */
     bool setIsNoDataExcept( QRect exceptRect );
 
     /**
@@ -469,8 +485,9 @@ class CORE_EXPORT QgsRasterBlock
      * method. This method has no effect for raster blocks with an explicit no data value set.
      *  \param row row index
      *  \param column column index
-     *  \since QGIS 2.10 */
-    void setIsData( int row, int column )
+     *  \since QGIS 2.10
+    */
+    void setIsData( int row, int column ) SIP_HOLDGIL
     {
       setIsData( static_cast< qgssize >( row )*mWidth + column );
     }
@@ -481,8 +498,9 @@ class CORE_EXPORT QgsRasterBlock
      * In this case it is possible to reset a pixel to flag it as having valid data using this
      * method. This method has no effect for raster blocks with an explicit no data value set.
      *  \param index data matrix index (long type in Python)
-     *  \since QGIS 2.10 */
-    void setIsData( qgssize index )
+     *  \since QGIS 2.10
+    */
+    void setIsData( qgssize index ) SIP_HOLDGIL
     {
       if ( mHasNoDataValue )
       {
@@ -551,7 +569,8 @@ class CORE_EXPORT QgsRasterBlock
      * \brief Print double value with all necessary significant digits.
      *         It is ensured that conversion back to double gives the same number.
      *  \param value the value to be printed
-     *  \returns string representing the value*/
+     *  \returns string representing the value
+     */
     static QString printValue( double value );
 
     /**
@@ -567,7 +586,8 @@ class CORE_EXPORT QgsRasterBlock
     /**
      * \brief Convert data to different type.
      *  \param destDataType dest data type
-     *  \returns true on success */
+     *  \returns TRUE on success
+    */
     bool convert( Qgis::DataType destDataType );
 
     /**
@@ -577,7 +597,7 @@ class CORE_EXPORT QgsRasterBlock
 
     /**
      * Sets the block data via an \a image.
-     * \returns true on success
+     * \returns TRUE on success
     */
     bool setImage( const QImage *image );
 
@@ -620,14 +640,14 @@ class CORE_EXPORT QgsRasterBlock
      * \see height
      * \since QGIS 2.10
      */
-    int width() const { return mWidth; }
+    int width() const SIP_HOLDGIL { return mWidth; }
 
     /**
      * Returns the height (number of rows) of the raster block.
      * \see width
      * \since QGIS 2.10
      */
-    int height() const { return mHeight; }
+    int height() const SIP_HOLDGIL { return mHeight; }
 
   private:
     static QImage::Format imageFormat( Qgis::DataType dataType );
@@ -637,7 +657,8 @@ class CORE_EXPORT QgsRasterBlock
      * Test if value is nodata comparing to noDataValue
      * \param value tested value
      * \param noDataValue no data value
-     * \returns true if value is nodata */
+     * \returns TRUE if value is nodata
+    */
     static bool isNoDataValue( double value, double noDataValue )
     {
       // TODO: optimize no data value test by memcmp()
@@ -650,12 +671,14 @@ class CORE_EXPORT QgsRasterBlock
     /**
      * Test if value is nodata for specific band
      * \param value tested value
-     * \returns true if value is nodata */
-    bool isNoDataValue( double value ) const;
+     * \returns TRUE if value is nodata
+    */
+    inline bool isNoDataValue( double value ) const;
 
     /**
      * Allocate no data bitmap
-     *  \returns true on success */
+     *  \returns TRUE on success
+    */
     bool createNoDataBitmap();
 
     /**
@@ -665,7 +688,8 @@ class CORE_EXPORT QgsRasterBlock
      *  \param srcDataType source data type
      *  \param destDataType dest data type
      *  \param size block size (width * height)
-     *  \returns block of data in destDataType */
+     *  \returns block of data in destDataType
+    */
     static void *convert( void *srcData, Qgis::DataType srcDataType, Qgis::DataType destDataType, qgssize size );
 
     // Valid
